function [D,H,Norm1H,Norm1HCol,Density,NrNZero,t_end,output] = ...
    DicLear_M(TImage,patch_size,s,Cof_Lambda,DSet,t,Outdisplay)

% The function loads the test images 
% Extract training patches from those images and learn a dictionary
% with the given parameters
% This function can also run without any input parameter and with the 
% default values defined in the code.

% Usage:
% [D,H,objec,Res,Res1,Res2,Res3,Res4,Res5,Density,NrNZero] = 
% DicLear_M(TImage,patch_size,s,Cof_Lambda,DSet,t)
%
% Input:
% TImage: Training test image ,
% patch_size: The extracted 2D training patch sizes and the patch size 
% in the  dictionary ,
% s: The number of dictionary elements,
% Cof_Lambda: the regularization parameter Lambda in the dictionary
% learning problem formulation
% DSet: the compact and convex set which the dictionary belongs to
% The options are: D_inf and D_2
% t: number of training patches that is considered in the dictionary 
% learning algorithm.
% Outdisplay: The display option on the screen, when set to 0 no display 
% on the screen, when set to 1 printes progress of the algorithm
% default = 0, 


% Output:
% D: The dictionary matrix for the corresponding lambda(s) of size p by s 
% where p=patch_size(1)*patch_size(1)
% H: The representation matrix for the corresponding lambda(s) of size:
% s by t
% objec: The objective function value for each iteration of the ADMM
% algorithm: 1/2||Y-DH||^2_F+\lambda||H||_{sum}
% Res: The residual value for each iteration of the ADMM
% algorithm: ||Y-DH||_F
% Res1,Res2,Res3,Res4,Res5: The stopping critera values at each iteration 
% of the ADMM algorithm
% Norm1H: ||H||_{sum}
% Norm1HCol: ||H||_{sum}/t
% Density: The density precentage of the matrix H
% NrNZero: The abslout number of nonzeros in the representation matrix H
% t_end: total time used by the ADMM algorithm to find a solution for a
% specific Lambda

% Copyright (C) Sara Soltani and Technical University of Denmark, 
% August 2015, ssol@dtu.dk, sarahsoltani@gmail.com

rng('default')
rng(500);      % Random seed

if nargin < 7 || isempty(Outdisplay)
    Outdisplay = 0;
end

if nargin < 6 || isempty(t)
    t = 10000;
end
if nargin < 5 || isempty(DSet)
    DSet = 'D_2';
end
if nargin < 4 || isempty(Cof_Lambda)
    Cof_Lambda = logspace(0.5,0.5,1);
end
if nargin < 3 || isempty(s)
    s = 300;
end
if nargin < 2 || isempty(patch_size)
    patch_size= [10 10];
end
if nargin < 1 || isempty(TImage)
    TImage= 'Peppers';
end

%% The test images


switch(TImage)
   case 'Peppers'
        TI = imread('Peppers.tif');
        TI= double(TI);
   case 'Matches'
         TI = imread('Matches.tif');
       TI= double(TI);
   case 'Binary'
         TI = imread('BinaryTI.tif');
         TI= double(TI);
   case 'Steel'
         TI = imread('Steel.tif');
         TI= double(TI);
    case 'Zirconium'
          TI = imread('Zirconium.tif');
         TI= double(TI);
         TI = filter2(fspecial('average',3),TI);
    case 'D53'
          TI = imread('D53.tif');
          TI= double(TI);
    otherwise
    warning('Please enter the correct name of the test image.')

         
end
 TI=(TI-min(TI(:)))./(max(TI(:))-min(TI(:))); 

% Extract patches from Training images


[Nrow,Ncol] = size(TI);

X=zeros(prod(patch_size),(Nrow-(patch_size(1)-1))*(Ncol-(patch_size(2)-1))); 
for i = 1:Nrow-(patch_size(1)-1)       % loop on all the rows
    for j = 1:Ncol-(patch_size(2)-1)   % loop on all the columns
           pattern = TI(i:i+patch_size(1)-1,j:j+patch_size(2)-1,1);   
           X(:,(i-1)*(Ncol-(patch_size(2)-1))+j)=pattern(:);               
    end
end

X=X(:,randperm(size(X,2)));

tol = 1e-2;       % tolerance
maxiter = 200;    % Maximum number of iterations

Nr_Lambda=length(Cof_Lambda);
m = patch_size(1)*patch_size(2);

D=zeros(m,s,Nr_Lambda);
H=zeros(s,t,Nr_Lambda);
Norm1H=zeros(Nr_Lambda,1);
Norm1HCol=zeros(Nr_Lambda,1);
t_end=zeros(Nr_Lambda,1);
Res=zeros(maxiter,Nr_Lambda);
Res1=zeros(maxiter,Nr_Lambda);
Res2=zeros(maxiter,Nr_Lambda);
Res3=zeros(maxiter,Nr_Lambda);
Res4=zeros(maxiter,Nr_Lambda);
Res5=zeros(maxiter,Nr_Lambda);

objec=zeros(maxiter,Nr_Lambda);
NrNZero=zeros(Nr_Lambda,1);
Density=zeros(Nr_Lambda,1);

% initialization
V = eye(s,t);
U = X(:,20:30:30*s);


Cof_rho=70;               % choose the parameter rho
                          % The algorithm's convergence depends on a
                          % carefull choice of this parameter which
                          % can be imperically determined 

for i=1:s
    switch(DSet)
        case 'D_inf'
              nrmU=norm(U(:,i),inf);  % for the first approach
        case 'D_2'                    % for the second approach
              nrmU=norm(U(:,i),2)/sqrt(patch_size(1)*patch_size(2)); 
    end
    if nrmU ~= 0
    U(:,i)=U(:,i)/nrmU;     
    V(i,:)=V(i,:)*nrmU;
    end
end

%%
for i=1:Nr_Lambda
     if Outdisplay==1
        disp(['The regularization parameter is: \lambda=',...
            num2str(Cof_Lambda)])
     end
     t0=cputime;
     switch(DSet)
        case 'D_inf'
             [ W,G,Rs1,Rs2,Rs3,Rs4,Rs5,fobjec,ResF] = Lea_Algorithm_Dinf...
             (X(:,1:t),tol,maxiter,s,Cof_Lambda(i),U,V,Cof_rho,Outdisplay);
        case 'D_2'
             [ W,G,Rs1,Rs2,Rs3,Rs4,Rs5,fobjec,ResF] = Lea_Algorithm_D2...
             (X(:,1:t),tol,maxiter,s,Cof_Lambda(i),U,V,Cof_rho,Outdisplay);
     end
 
 
     t_end(i,1)=cputime-t0;

    
     Res1(:,i)=Rs1;
     Res2(:,i)=Rs2;
     Res3(:,i)=Rs3;
     Res4(:,i)=Rs4;
     Res5(:,i)=Rs5;
     Res(:,i)=ResF;
     objec(:,i)=fobjec;
     
     
     D(:,:,i)=W;
     H(:,:,i)=G;
     Norm1HCol(i,1)=norm(H(:),1)/size(H,2);
     Norm1H(i,1)=norm(H(:),1);
     NrNZero(i,1)=length(find(H));
     Density(i,1)=NrNZero/length(H(:))*100;
     
end

 output = struct('Residual',Res,'Objective',objec,'StopCr1',Res1,...
     'StopCr2',Res2,'StopCr3',Res3,'StopCr4',Res4,'StopCr5',Res5);
end


