function op = prox_nuclearposl1( q, LARGESCALE )
% BY SARA SOLTANI
% PROX_NUCLEARPOSL1  Nuclear norm + l1 norm regularization
% we compute  the prox operator for ||.||_1+||.||_*
% iteratively using a Dykstra-like proximal algorithm from 
% [CombettesPesquet2011] where prox operators of ||.||_1 and ||.||_* are
% consecutively computed in an iterative scheme.
% Usage:   OP = PROX_NUCLEARPOSL1( q ) 
%    q is optional; if omitted, q=1 is assumed. But if q is supplied, 
%    it must be a positive real scalar.
%
%    OP = PROX_NUCLEARPOSL1( q, LARGESCALE )
%       uses a Lanczos-based SVD if LARGESCALE == true,
%       otherwise it uses a dense matrix SVD
%
%    CALLS = PROX_NUCLEARPOSL1( 'reset' )
%       resets the internal counter and returns the number of function
%       calls
%
% This proximity operator implementation uses aapproach that does exploit 
% a priori knowledge that X are low rank and sparse. This codes is
% insipred by implementations prox operators in TFOCS-1.3.1 
% (http://cvxr.com/tfocs/)
% e.g. see prox_nuclear.m and prox_l1pos.m in the TFOCS folder

% Copyright (C) Sara Soltani and Technical University of Denmark, 
% August 2015, ssol@dtu.dk, sarahsoltani@gmail.com


if nargin == 1 && strcmpi(q,'reset')
    op = prox_nuclear_impl;
    return;
end

if nargin == 0,
	q = 1;
elseif ~isnumeric( q ) || ~isreal( q ) || numel( q ) ~= 1 || q <= 0,
	error( 'Argument must be positive.' );
end
if nargin < 2, LARGESCALE = []; end

% clear the persistent values:
prox_nuclear_impl();

op = @(varargin)prox_nuclear_impl( q, LARGESCALE, varargin{:} );

end % end of main function

function [ v, X ] = prox_nuclear_impl( q, LARGESCALE, X, t )
persistent oldRank
persistent nCalls
if nargin == 0, oldRank = []; v = nCalls; nCalls = []; return; end
if isempty(nCalls), nCalls = 0; end

ND = (size(X,2) == 1);
  
% ND = ~ismatrix(X);
if ND, % X is a vector, not a matrix, so reshape it 
    sx = size(X);
    X = reshape( X, prod(sx(1:end-1)), sx(end) );
end
% initialization 

if nargin > 3 && t > 0,
%  size(X)
XX=X;
P=zeros(size(X));
Q=P;

for i=1:100
 % first step
    X=XX+P;
 % first step Cont   
 
    
    if ~isempty(LARGESCALE) % inherited from parent
        largescale = LARGESCALE;
    else
        largescale = ( numel(X) > 100^2 ) && issparse(X);
    end
    tau = q*t;
    nCalls = nCalls + 1;
    
%     fprintf('ranks: ');
    if ~largescale
        [U,S,V] = svd( full(X), 'econ' );
    else
        % Guess which singular value will have value near tau:
        [M,N] = size(X);
        if isempty(oldRank), K = 10;
        else K = oldRank + 2;
        end
        
        ok = false;
        opts = [];
        opts.tol = 1e-10; % the default in svds
        opt  = [];
        opt.eta = eps; % makes compute_int slow
%         opt.eta = 0;  % makes reorth slow
        opt.delta = 10*opt.eta;
        while ~ok
            K = min( [K,M,N] );
            if exist('lansvd','file')
                [U,S,V] = lansvd(X,K,'L',opt );
            else
                [U,S,V] = svds(X,K,'L',opts);
            end
            ok = (min(diag(S)) < tau) || ( K == min(M,N) );
%             fprintf('%d ',K );
            if ok, break; end
%             K = K + 5;
            K = 2*K;
            if K > 10
                opts.tol = 1e-6;
            end
            if K > 40
                opts.tol = 1e-4;
            end
            if K > 100
                opts.tol = 1e-1;
            end
            if K > min(M,N)/2
%                 disp('Computing explicit SVD');
                [U,S,V] = svd( full(X), 'econ' );
                ok = true;
            end
        end
        oldRank = length(find(diag(S) > tau));
    end
    s  = diag(S) - tau;
    tt = s > 0;
    s  = s(tt,:);
    
%     fprintf('\n')';
%     fprintf('rank is %d\n', length(tt) );
    
    % Check to make sure this doesn't break existing code...
    if isempty(s),
%         X(:) = 0;  % this line breaks the packSVD version
        Y = tfocs_zeros(X);
    else
        Y = U(:,tt) * bsxfun( @times, s, V(:,tt)' );
    end

   
v = q * sum(s);

% second step
P=X-Y;
% third step


    
XX   = max( 0, Y+Q - t*q);

% forth step
Q=Y+Q-XX ;

 if norm(XX-Y)<= 0.001 % && norm(q) <= tol
      break
 end


end 

X = XX;
%size(v)
else 
 
 s = svd(full(X)); % could be expensive!

v = q * sum(s);

v  = max( 0, v);
end
if ND, 
    X = reshape( X, sx ); 
end

end

